
/**************************************************************************
  sumtext.c

  1) Description
    
  A column-summing DLL extension package for Premia Corp.'s CodeWright
  editor. Select (hilight) a portion of numeric text in a buffer, and run
  this function to calculate the sum.  Result can be displayed on the
  status line and/or copied to the clipboard.

  2) Author, history and acknowledgements

  1993/09/06  Copyright (c) David Landgren

  Bug reports, compiler ports, praise & flames should be sent to:
    Internet: <david.landgren@oecd.fr>
    or        <dlandgren@bix.com>
    or x400:  <c=fr;a=atlas;p=oecd;o=oecd;s=landgren;g=david>

  Warm thanks are extended to Brad Smith <b.p.smith@bix.com> for helping
  out with this.

  Compiled and tested using Microsoft C v8.0

  3) Making it

  This file can be compiled as a stand-alone DLL, by defining the
  preprocessor symbol `STANDALONE_DLL', otherwise it can be used as part of
  a larger DLL tool-kit.  In the case of a stand-alone DLL, you'll want to
  create a module definition file modelled on the sample supplied with
  CodeWright.  The EXPORT section will look like:

  =============
  EXPORTS
    WEP              @1 RESIDENTNAME
    SUMSELECTEDTEXT  @2
  =============

  The distributed sample makefile can compile this file with a little work.

  4) Restrictions of use

  This file shall be distributed according to the terms laid down in the
  Perl Artistic License.  The full text is not reproduced here (mail me if
  you want a copy of it), but the relevant points are:

    1. You may make and give away verbatim copies of the source form of
    the Standard Version of this Package without restriction, provided that
    you duplicate all of the original copyright notices and associated
    disclaimers.

    2. You may apply bug fixes, portability fixes and other modifications
    derived from the Public Domain or from the Copyright Holder.  A
    Package modified in such a way shall still be considered the Standard
    Version.

 *************************************************************************/

#include <windows.h>    /* for wsprintf() */
#include "exports.h"
#include "cwstart.h"
#include <stdlib.h>     /* for atol() */
#include <ctype.h>      /* for isdigit() */


/* private routines */
long DLL _sum_numerics( void );

    /* this routine is a blatant plagiarisation of _do_block() in
     * startup/util.c.  I'd would have used that, but it is not exported,
     * nor does permit read-only buffers.  As I do not modify the buffer
     * I just chopped all of that stuff out, and folded in the callback.
     */

LIBMAIN    /* defines the LibMain() function for Borland C */


/* CodeWright DLL initialiser */
void DLL _init()
{
    LibExport( "long SumSelectedText WORD" );
}

#define DO_PRINT  1
#define DO_COPY   2
#define DO_REMOVE 4
#define DO_SILENT 8

long DLL SumSelectedText( WORD flag )
{
    /* INPUT
     * flag -- OR in the following bits for the desired behaviour
     * ====
     *   1 - result is printed on status line
     *   2 - result is copied to clipboard or current scrap
     *   4 - selection is removed after calculation
     *   8 - no error messages are displayed
     *
     * RETURNS
     * =======
     * long  - the result of the sum.  Could be zero.
     *         Set to zero if no buffer or no selection is currently active
     *
     * NOTES
     * =====
     * The buffer is not modified.
     *
     * An attempt is made to skip over non-digit characters on the
     * beginning of each line.  The remaining string is then passed to
     * atol(), in the standard library, to do the conversion.  As such,
     * only the first number on the line will be taken into account,
     * which, as it happens, is the behaviour I want.
     *
     * I hope I succeded in creating a file that is both C and C++
     * compilable.
     *
     * I use longs.  You may want to use doubles.
     */

    long    sum    = 0L;
    HBUFFER curbuf = BufQCurrentBuffer();

    if( !curbuf )
    {
        if( !(flag & DO_SILENT) )
        {
            MsgMessage( "No current buffer." );
        }
        return 0L;
    }

    if( SELECTION_NONE == MarkQSelType() )
    {
        if( !(flag & DO_SILENT) )
        {
            MsgMessage( "No selection is currently defined in this buffer." );
        }
        return 0L;
    }

    /* go calculate it */
    sum = _sum_numerics();

    /* print it out on the status line ? */
    if( flag & DO_PRINT )
    {
        char msg[20];

        wsprintf( msg, "sum is %ld", sum );
        MsgMessage( msg );
    }

    /* copy it to the current scrap or clipboard ? */
    if( flag & DO_COPY )
    {
        /* Format the binary num into its ASCII equivalent.  There appears
         * to be no way to directly insert a string to the current scrap or
         * clipboard, so we open a system buffer, and paste the string into
         * an exclusive selection, and cut that to the scrap or clipboard,
         * depending on the config info.  This lets us use the routine on
         * read-only buffers, otherwise I'd insert the text into the
         * current buffer.
         */

        HBUFFER sumbuf;
        char msg[20];

        wsprintf( msg, "%ld", sum );
        sumbuf = BufCreate( "@@@colsum_tmpbuf@@@", NULL, TRUE );
        if( sumbuf )
        {
            BufSetCurrentBuffer( sumbuf );

            MarkBeginSel( SELECTION_EXCLUSIVE );
            BufInsertStr( (LPSTR)msg );
            MarkEndSel();

            if( SysQFlags() & SYSFLAG_CLIPBOARD )
            {
                ClipboardCut();          
            }
            else
            {
                BufCutToScrap();
            }

            BufSetCurrentBuffer( curbuf );
            BufDelBuffer( sumbuf );
        }
    }

    /* remove the selection ? */
    if( flag & DO_REMOVE )
    {
        MarkRemoveSel();
    }

    return sum;
}

long DLL _sum_numerics( void )
{
    /* really is just startup/util.c[ _do_block() ] in disguise... */

    long  lcol, rcol;
    long  tline, bline;
    int   type;
    long  sum = 0L;

    SysWaitStart();

    type = MarkCoordinates( &tline, &lcol, &bline, &rcol );
    MarkSavePos();

    if (MovAbs( tline, lcol ) 
        || ((tline == (long)BufQCurrentLine()) 
                && (lcol == (long)BufQCurrentCol())))
    {
        if (type != SELECTION_COLUMN)
            lcol = 1;

        do
        {
            long size;

            MarkBeginSel( SELECTION_INCLUSIVE );
            MovAbs( tline, (tline == bline || type == SELECTION_COLUMN)
                    ? rcol : 0x7FFF );

            MarkSwapSel();

            size = MarkQSelSize( FALSE );
            MarkRemoveSel();

            {
                LPSTR str;
                LPSTR ptr = str = BufReadStr( size );

                /* step over non-digits - but don't use the macro */
                while( *ptr && !(isdigit)( *ptr ))
                {
                    ++ptr;
                }

                if( *ptr )
                {
                    sum += atol( ptr );
                }

                StrFree( str );
            }

            tline++;

        } while ((tline <= bline) && MovAbs( tline, lcol ));
    }

    MarkRestorePos();
    SysWaitEnd();

    return sum;
}

/* end colsum.c */
